// main.js - put your code here
var conf = {
	updateTime : 40,
	// max acceleration
	maxAcc : 7
};

function Point(x, y) {
	this.x = x ? x : 0;
	this.y = y ? y : 0;
}

var bubble = (function() {
	var hLevel, vLevel, sLevel, bubbleX, bubbleY, bubbleS, hasHorizontal = true, hasVertical = true, hasSpherical = true, maxXY, maxS, maxAccRatio, updateFactor = conf.updateTime / 1000, calibration = new Point, pos = new Point, destination = new Point, velocity = new Point, screenWidth, screenHeight, aspectRatio, accX, accY, accZ, updateOrientationInterval = 0;
	var shiftRight = [ '-240px', '-210px', '-180px', '-150px' ];
	var shiftLeft = [ '0', '-30px', '-60px', '-90px' ];
	return {
		init : function() {
			aspectRatio = document.body.clientWidth
					/ document.body.clientHeight;
			hLevel = document.getElementById('horizontal-level');
			vLevel = document.getElementById('vertical-level');
			sLevel = document.getElementById('spherical-level');
			bubbleX = document.getElementById('bubbleX');
			bubbleY = document.getElementById('bubbleY');
			bubbleS = document.getElementById('bubbleS');
			maxXY = 268;
			maxS = 250;
			maxAccRatio = maxXY / conf.maxAcc;
			if (hasSmallScreen()) {
				updateOrientation();
				updateOrientationInterval = window.setInterval(
						updateOrientation, 1000);
			}
		},
		update : update,
		updateOrientation : updateOrientation,

		calibrate : function() {
			calibration.x = accX;
			calibration.y = accY;
		}
	};

	function updateOrientation() {
		if (!hasSmallScreen()) { // FIXME WM bug (screen size has 128x128 after resume from minimized state)
			hasHorizontal = true;
			hasVertical = true;
			hasSpherical = true;
			hLevel.style.display = 'block';
			vLevel.style.display = 'block';
			sLevel.style.display = 'block';
			clearInterval(updateOrientationInterval);
			updateOrientationInterval = 0;
			return;
		}
		if (Math.abs(accX) > 7 && aspectRatio < 1) {
			hasHorizontal = false;
			hasVertical = true;
			hasSpherical = false;
			hLevel.style.display = 'none';
			vLevel.style.display = 'block';
			sLevel.style.display = 'none';
		}
		else if (Math.abs(accY) > 7) {
			hLevel.style.display = 'block';
			vLevel.style.display = 'none';
			sLevel.style.display = 'none';
			hasHorizontal = true;
			hasVertical = false;
			hasSpherical = false;
		}
		else if (Math.abs(accX) < 2 && Math.abs(accY) < 2 || Math.abs(accX) > 7
				&& aspectRatio > 1) {
			hasHorizontal = false;
			hasVertical = false;
			hasSpherical = true;
			hLevel.style.display = 'none';
			vLevel.style.display = 'none';
			sLevel.style.display = 'block';
		}
	}

	function updateAccelerometer() {
		// switching accelerometer values to fix ratio problem
		if (aspectRatio > 1) {
			accY = Widget.Device.DeviceStateInfo.AccelerometerInfo.xAxis;
			accX = -Widget.Device.DeviceStateInfo.AccelerometerInfo.yAxis;
		}
		else {
			accX = Widget.Device.DeviceStateInfo.AccelerometerInfo.xAxis;
			accY = Widget.Device.DeviceStateInfo.AccelerometerInfo.yAxis;
		}
		accZ = Widget.Device.DeviceStateInfo.AccelerometerInfo.zAxis;
	}

	/**
	 * Update velocity of bubbles
	 */
	function update() {
		updateAccelerometer();
		if (Math.abs(accX) < 0.1) {
			accX = 0;
		}
		if (Math.abs(accY) < 0.1) {
			accY = 0;
		}
		destination.x = (accX - calibration.x) * maxAccRatio;
		destination.y = (accY - calibration.y) * maxAccRatio;
		velocity.x += (destination.x - pos.x) / 2 - velocity.x / 4;
		velocity.y += (destination.y - pos.y) / 2 - velocity.y / 4;
		if (Math.abs(velocity.x) < 0.01) {
			velocity.x = 0;
		}
		if (Math.abs(velocity.y) < 0.01) {
			velocity.y = 0;
		}
		move(velocity.x * updateFactor, velocity.y * updateFactor);
	}
	/**
	 * Move every bubble for specific pixels
	 */
	function move(x, y) {
		pos.x += x;
		pos.y += y;

		if (pos.x > maxXY / 2) {
			pos.x = maxXY / 2;
		}
		else if (pos.x < -maxXY / 2) {
			pos.x = -maxXY / 2;
		}

		if (pos.y > maxXY / 2) {
			pos.y = maxXY / 2;
		}
		else if (pos.y < -maxXY / 2) {
			pos.y = -maxXY / 2;
		}

		if (hasHorizontal) {
			moveHorizontal();
		}
		if (hasVertical) {
			moveVertical();
		}
		if (hasSpherical) {
			moveSpherical();
		}
	}

	function moveHorizontal() {
		var newX = Math.max(9, Math.min(-Math.floor(pos.x) + maxXY / 2 - 14,
				maxXY - 42)), shiftX = '50%', shiftY = '50%';
		if (newX < 13) {
			shiftX = shiftLeft[~~newX - 9];
		}
		else if ((maxXY - newX - 42) < 4) {
			shiftX = shiftRight[~~(maxXY - newX - 42)];
		}

		if (shiftX === '50%') {
			if (pos.y > 0) {
				shiftY = shiftRight[3 - Math.min(3, ~~(pos.y / 10))];
			}
			else if (pos.y < 0) {
				shiftY = shiftLeft[3 - Math.min(3, ~~(-pos.y / 10))];
			}
		}
		else {
			if (pos.y > 40) {
				shiftY = '100%';
			}
			else if (pos.y < -40) {
				shiftY = '0';
			}
		}
		bubbleX.style.backgroundPosition = shiftX + ' ' + shiftY;
		bubbleX.style.left = newX;
	}

	function moveVertical() {
		var newY = Math.max(9, Math.min(pos.y + maxXY / 2 - 14, maxXY - 44)), shiftY = '50%', shiftX = '50%';
		if (newY < 13) {
			shiftY = shiftLeft[~~newY - 9];
		}
		else if ((maxXY - newY - 44) < 4) {
			shiftY = shiftRight[~~(maxXY - newY - 44)];
		}
		if (shiftY === '50%') {
			if (pos.x < 0) {
				shiftX = shiftRight[3 - Math.min(3, ~~(-pos.x / 20))];
			}
			else if (pos.x > 0) {
				shiftX = shiftLeft[3 - Math.min(3, ~~(pos.x / 20))];
			}
		}
		else {
			if (pos.x > 40) {
				shiftX = '0';
			}
			else if (pos.x < -40) {
				shiftX = '100%';
			}
		}

		bubbleY.style.backgroundPosition = shiftX + ' ' + shiftY;
		bubbleY.style.top = newY;
	}

	function moveSpherical() {
		var r = (maxS - 95) / 2; // radius without margin
		var s = Math.min(1, r
				/ (Math.sqrt(Math.pow(pos.x, 2) + Math.pow(pos.y, 2))));
		bubbleS.style.left = -pos.x * s + maxS / 2 - 17;
		bubbleS.style.top = pos.y * s + maxS / 2 - 17;
	}

	function hasSmallScreen() {
		return Math.min(document.body.clientWidth, document.body.clientHeight) <= 420
				|| Math.max(document.body.clientWidth,
						document.body.clientHeight) <= 480;
	}
})();

var app = (function() {
	var intervalId = 0, calibrate, hold;
	return {
		init : function() {
			var calibrateButton = document.getElementById('calibrate');
			bubble.init();
			intervalId = window.setInterval(bubble.update, conf.updateTime);

			calibrateButton.onclick = function() {
				calibrateButton.className = 'calibrate active';
				window.setTimeout(function() {
					calibrateButton.className = 'calibrate';
				}, 500);
				bubble.calibrate();
			}
			document.getElementById('hold').onclick = toggleHold;
			window.addEventListener("resize", function() {
				bubble.init();
			}, false);
		}
	}

	function toggleHold() {
		if (intervalId) {
			document.getElementById('hold').className = 'hold active';
			clearInterval(intervalId);
			intervalId = 0;
		}
		else {
			document.getElementById('hold').className = 'hold';
			intervalId = window.setInterval(bubble.update, conf.updateTime);
		}
	}
})();
window.addEventListener('DOMContentLoaded', function() {
	app.init();
}, false);
